@@ -152,7 +152,7 @@ namespace xt
         {
             const auto& de = e.derived_cast();
             R ev;
-            ev.resize({de.size()});
+            ev.resize({static_cast<typename R::shape_type::value_type>(de.size())});
 
             std::copy(de.cbegin(), de.cend(), ev.begin());
             std::sort(ev.begin(), ev.end());
@@ -279,15 +279,15 @@ namespace xt
             {
                 n_iters = std::accumulate(data.shape().begin(), data.shape().end() - 1,
                                           std::size_t(1), std::multiplies<>());
-                data_secondary_stride = data.shape(data.dimension() - 1);
-                inds_secondary_stride = inds.shape(inds.dimension() - 1);
+                data_secondary_stride = static_cast<std::ptrdiff_t>(data.shape(data.dimension() - 1));
+                inds_secondary_stride = static_cast<std::ptrdiff_t>(inds.shape(inds.dimension() - 1));
             }
             else
             {
                 n_iters = std::accumulate(data.shape().begin() + 1, data.shape().end(),
                                           std::size_t(1), std::multiplies<>());
-                data_secondary_stride = data.shape(0);
-                inds_secondary_stride = inds.shape(0);
+                data_secondary_stride = static_cast<std::ptrdiff_t>(data.shape(0));
+                inds_secondary_stride = static_cast<std::ptrdiff_t>(inds.shape(0));
             }
 
             auto ptr = data.data();
@@ -768,31 +768,6 @@ namespace xt
             using type = xtensor<std::size_t, N - 1>;
         };
 
-        template <class IT, class F>
-        inline std::size_t cmp_idx(IT iter, IT end, std::ptrdiff_t inc, F&& cmp)
-        {
-            std::size_t idx = 0;
-            auto min = *iter;
-            iter += inc;
-            for (std::size_t i = 1; iter < end; iter += inc, ++i)
-            {
-                if (cmp(*iter, min))
-                {
-                    min = *iter;
-                    idx = i;
-                }
-            }
-            return idx;
-        }
-
-        template <layout_type L, class E, class F>
-        inline xtensor<std::size_t, 0> arg_func_impl(const E& e, F&& f)
-        {
-            return cmp_idx(e.template begin<L>(),
-                           e.template end<L>(), 1,
-                           std::forward<F>(f));
-        }
-
         template <layout_type L, class E, class F>
         inline typename argfunc_result_type<E>::type
         arg_func_impl(const E& e, std::size_t axis, F&& cmp)
@@ -804,7 +779,10 @@ namespace xt
 
             if (e.dimension() == 1)
             {
-                return arg_func_impl<L>(e, std::forward<F>(cmp));
+                auto begin = e.template begin<L>();
+                auto end = e.template end<L>();
+                std::size_t i = static_cast<std::size_t>(std::distance(begin, std::min_element(begin, end)));
+                return xtensor<size_t, 0>{i};
             }
 
             result_shape_type alt_shape;
@@ -857,14 +835,19 @@ namespace xt
     {
         using value_type = typename E::value_type;
         auto&& ed = eval(e.derived_cast());
-        return detail::arg_func_impl<L>(ed, std::less<value_type>());
+        auto begin = ed.template begin<L>();
+        auto end = ed.template end<L>();
+        std::size_t i = static_cast<std::size_t>(std::distance(begin, std::min_element(begin, end)));
+        return xtensor<size_t, 0>{i};
     }
 
     /**
-     * Find position of minimal value in xexpression
+     * Find position of minimal value in xexpression.
+     * By default, the returned index is into the flattened array.
+     * If `axis` is specified, the indices are along the specified axis.
      *
      * @param e input xexpression
-     * @param axis select axis (or none)
+     * @param axis select axis (optional)
      *
      * @return returns xarray with positions of minimal value
      */
@@ -882,14 +865,19 @@ namespace xt
     {
         using value_type = typename E::value_type;
         auto&& ed = eval(e.derived_cast());
-        return detail::arg_func_impl<L>(ed, std::greater<value_type>());
+        auto begin = ed.template begin<L>();
+        auto end = ed.template end<L>();
+        std::size_t i = static_cast<std::size_t>(std::distance(begin, std::max_element(begin, end)));
+        return xtensor<size_t, 0>{i};
     }
 
     /**
      * Find position of maximal value in xexpression
+     * By default, the returned index is into the flattened array.
+     * If `axis` is specified, the indices are along the specified axis.
      *
      * @param e input xexpression
-     * @param axis select axis (or none)
+     * @param axis select axis (optional)
      *
      * @return returns xarray with positions of maximal value
      */
