@@ -334,7 +334,6 @@ namespace xt
     /***********************************
      * resize_container implementation *
      ***********************************/
-
     template <class C>
     inline bool resize_container(C& c, typename C::size_type size)
     {
@@ -638,11 +637,16 @@ namespace xt
      * xtrivial_default_construct implemenation *
      ********************************************/
 
+#if defined(_GLIBCXX_RELEASE) && _GLIBCXX_RELEASE >= 7
+// has_trivial_default_constructor has not been available since libstdc++-7.
+#define XTENSOR_GLIBCXX_USE_CXX11_ABI 1
+#else
 #if defined(_GLIBCXX_USE_CXX11_ABI)
 #if _GLIBCXX_USE_CXX11_ABI || (defined(_GLIBCXX_USE_DUAL_ABI) && !_GLIBCXX_USE_DUAL_ABI)
 #define XTENSOR_GLIBCXX_USE_CXX11_ABI 1
 #endif
 #endif
+#endif
 
 #if !defined(__GNUG__) || defined(_LIBCPP_VERSION) || defined(XTENSOR_GLIBCXX_USE_CXX11_ABI)
 
@@ -871,7 +875,7 @@ namespace xt
     template <class E>
     struct get_rank<E, decltype((void)E::rank, void())>
     {
-        constexpr static std::size_t value= E::rank;
+        constexpr static std::size_t value = E::rank;
     };
 
     /******************
